function sl = GetRandomConstraint(nb,ML,CL,linked,init)
% Get one random constraints not already supervised.
%
% Input:
%   nb:     Number of constraints requested
%   ML:     Must link matrix
%   CL:     Cannot link matrix
%   linked: Select only linked link to already supervised
%   init:   If present, init persistent data
%
% Output:
%   sl:     nbc x 2, the selected pairs
% 
% Persistent
%   rts:    2 x (1+n(n-1)/2) matrix, where colums are the indexes of all
%           coefficient of matrix tri sup of size n (in random order)
persistent rts;
if ((nargin==5)&&(strcmp(init,'INIT')))
    n = size(ML,1);
    randp = randperm(n*n);
    randindex = [floor(randp/n);mod(randp,n)]+1;
    trisup = randindex(1,:)<randindex(2,:);
    rts = [2,randindex(1,trisup);0,randindex(2,trisup)];
%    rts = [2,1,1,1,1,2,3,4,5,3;0,2,3,5,6,3,4,5,6,5]
else
    idx = rts(1,1);
    sl = zeros(nb,2);
    i = 0;
    move = true;
    while(idx<=size(rts,2))
        row = rts(1,idx);
        col = rts(2,idx);
        if ((ML(row,col)==0)&&(CL(row,col)==0))
            if ((~linked)||nnz(ML(row,:))||nnz(ML(col,:))||nnz(CL(row,:))||nnz(CL(col,:))||(idx<=nb+1))
                if (move)
                    idxref = idx;
                end
                i = i + 1;
                sl(i,:) = [row,col];
                if (i>=nb)
                    break;
                end
            else
                move = false;
                idxref = idx;
            end
        end
        idx = idx + 1;
    end
    if (idx>size(rts,2))
        sl = sl(1:i,:);
    end
    if (~isempty(sl))
        rts(1,1) = idxref + 1;
    end
end
