%function Main()
clear;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% PARAMETERS TO ADJUST
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
addpath('../cosc_v1_1'); %COSC directory
dirResult = '../result'; %Result directory where result file are copied
dirData = '../data';     %Directory of the file data

%fileData = '2clusters_1StronglySeparated.csv';
%fileData = '2clusters_2Separated.csv';
%fileData = '2clusters_3Contiguous.csv';
%fileData = '2clusters_4PartiallyMixed.csv';
%fileData = '2clusters_5Random.csv';
fileData = 'sonar.data';

GT_COL = -1;       %GROUND TRUTH COLUMN : 1 for first, 2 for second, -1 for last
K = 2;             %2 CLUSTERS (This code only work with 2 clusters !)
INDEX = 'adjrand'; %INDEX = 'jaccard';

METHOD = 'sple';  %For Spectral Learning method
%METHOD = 'clsl'; %For COSC used like Spectral Learning
%METHOD = 'cosc'; %For COSC original method

PROPAGATION = true;   %true/false
LINKED_PAIRS = false; %true/false

NB_CONSTRAINT = 1;    %Number of constaint to supervise between 2 clustering
NB_SUPERVISION = 208; %Number of clustering to process
NB_INCREMENT = 2;     %Number of total execution to obtain an average quality

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%% END OF PARAMETERS TO ADJUST
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[empty,fileDataName,fileDataExt] = fileparts(fileData);
fileResult = [ 'q_' fileDataName '_' num2str(K) '_' INDEX '_' METHOD '_' num2str(PROPAGATION) num2str(LINKED_PAIRS) '_' num2str(NB_CONSTRAINT) 'x' num2str(NB_SUPERVISION) 'x' num2str(NB_INCREMENT) '.mat' ];
fileResultIter = [ 'iter_' fileResult ];
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
tmp = csvread([dirData '/' fileData ]); %Load data
N = size(tmp,1); %Number of objects
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
NB_KNN = ceil(log(N));
if (GT_COL == -1)
    clusters_gt = tmp(:,end); %Ground truth
    tmpdata = tmp(:,1:end-1);   %Data coordinates
else
    clusters_gt = tmp(:,GT_COL); %Ground truth
    tmpdata = tmp(:,[1:GT_COL-1,GT_COL+1:end]); %Data coordinates
end
data = tmpdata-(ones(size(tmpdata,1),1)*mean(tmpdata,1)); %Data standardisation
data = data./(ones(size(tmpdata,1),1)*((std(tmpdata,1)==0) +std(tmpdata,1))); %Data standardisation
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for increment=1:NB_INCREMENT
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%   
%     nb = 0;
%     if (exist([dirResult '/' fileResult], 'file') == 2)
%         load([dirResult '/' fileResult]); %A previous execution already done
%         if (nb>=NB_INCREMENT) %nb is the number of increment already done
%             disp('A previous execution already done (result file already exists) and total number of increment reach');
%             break;
%         end
%     end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    isPerfect = 0;
    supermax = 0;
    [index,distance] = knnsearch(data,data,'K',NB_KNN+1,'IncludeTies',true, 'NSMethod','kdtree');
    tmp = zeros(N,N);
    for j=1:N
        for k=1:NB_KNN
            if (index{j}(k)~=j)
                tmp(j,index{j}(k)) = 0.1 + (exp(-(distance{j}(k)^2)/2))*0.8;
                tmp(index{j}(k),j) = tmp(j,index{j}(k));
            end
        end
    end
%   hist(tmp(tmp>0));
    W = sparse(tmp);
    D = sum(W,2);
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    ML = sparse(N,N);
    CL = sparse(N,N);
    x = zeros(NB_SUPERVISION,1);
    q = ones(NB_SUPERVISION,1);
    GetRandomConstraint(0,ML,CL,LINKED_PAIRS,'INIT');
    for super=1:NB_SUPERVISION
        disp('Supervision:');
        disp(super);
        [tmp1,tmp2] = find(ML);
        ml=[tmp1,tmp2];
        [tmp1,tmp2] = find(CL);
        cl=[tmp1,tmp2];
        tic;
        if ((strcmp(METHOD,'sple'))||(strcmp(METHOD,'clsl')))
            W = max(ML,W);
            W = min(sparse(1-CL),W);
            D = sum(W,2);
            if (strcmp(METHOD,'sple'))
                disp('Spectral Learning...');
                clusters = SpectralClustering(W, K, 3);
            else
                disp('COSC used like Spectral Learning...');
                [cut, clusters, viols] = cosc(W, D, K, [], []);
            end
        elseif (strcmp(METHOD,'cosc'))
            disp('Cosc...');
            [cut, clusters, viols] = cosc(W, D, K, ml, cl);
        else
            disp('ERROR with METHOD...');
        end
        toc;
        if (strcmp(INDEX,'adjrand'))
            q(super) = adjrand(clusters+1, clusters_gt+1);
        elseif (strcmp(INDEX,'jaccard'))
            q(super) = jaccard(clusters+1, clusters_gt+1);
        else
            disp('ERROR with INDEX...');
        end
        if (q(super)==1 ) % If 3 consecutive perfect partition => stop (TIME GAIN !!!)
            if (isPerfect<2)
                isPerfect = isPerfect + 1;
            else
                break;
            end
        else
            isPerfect = 0;
        end
        if super>1
            x(super) = x(super-1) + NB_CONSTRAINT;
        end
        if super<NB_SUPERVISION
            tic;
            for i=1:NB_CONSTRAINT
                disp('One random constraint...');
                sl = GetRandomConstraint(1,ML,CL,LINKED_PAIRS);
                disp('Supervision...');
                [ML,CL,MLadd,CLadd] = Supervise(sl,clusters_gt,ML,CL);
                if (PROPAGATION==true)
                    disp('Propagation...');
                    [ML,CL,qtmp] = Propagation(ML,CL,MLadd,CLadd,K,PROPAGATION);
                end
            end
            toc;
        end
       plot(x(1:super),q(1:super),'-k');
       drawnow;
        save([[dirResult '/' fileResultIter]],'super','x','q');
    end
    nb = 0;
    qw = 0;
    if (exist([dirResult '/' fileResult], 'file') == 2)
        load([dirResult '/' fileResult]);
    end
    if (supermax<super)
        supermax = super;
        xmax = x;
    end
    q = (nb*qw + q)./(nb+1);
    nb = nb +1;
    qw = q;
    save([dirResult '/' fileResult],'nb','qw','supermax','xmax');
    plot(xmax(1:supermax-1),qw(1:supermax-1),'-k');
    drawnow;
end
