function jaccard=jaccard(u,v)

%function jaccard=jaccard(u,v)
%
% Computes the Jaccard index to assess the quality of a clustering.
% Perfectly random clustering returns the minimum score of 0, perfect
% clustering returns the maximum score of 1.
%
%INPUTS
% u = the labeling as predicted by a clustering algorithm
% v = the true labeling
%
%OUTPUTS
% jaccard = the Jaccard index
%
%
%Author: Nicolas Voiron, may 2015.

n=length(u);
dn=n*(n-1)/2;
y1 = pdist(u(:),'ham');
y2 = pdist(v(:),'ham');
ad=sum((y1'==y2'));
a = sum((y1'==0).*(y2'==0));
jaccard = a/(dn-ad+a);

